@@ -24,6 +24,17 @@
 #define PANGO_SCALE_26_6 (PANGO_SCALE / (1<<6))
 #define PANGO_UNITS_26_6(d) (PANGO_SCALE_26_6 * (d))
 
+/**
+ * pango_ot_buffer_new
+ * @font: a #PangoFcFont
+ *
+ * Creates a new #PangoOTBuffer for the given OpenType font.
+ *
+ * Return value: the newly allocated #PangoOTBuffer, which should
+ *               be freed with pango_ot_buffer_destroy().
+ *
+ * Since: 1.4
+ **/ 
 PangoOTBuffer *
 pango_ot_buffer_new (PangoFcFont *font)
 {
@@ -49,6 +60,14 @@ pango_ot_buffer_new (PangoFcFont *font)
   return buffer;
 }
 
+/**
+ * pango_ot_buffer_destroy
+ * @buffer: a #PangoOTBuffer
+ *
+ * Destroys a #PangoOTBuffer and free all associated memory.
+ *
+ * Since: 1.4
+ **/ 
 void
 pango_ot_buffer_destroy (PangoOTBuffer *buffer)
 {
@@ -57,6 +76,14 @@ pango_ot_buffer_destroy (PangoOTBuffer *buffer)
   g_free (buffer);
 }
 
+/**
+ * pango_ot_buffer_clear
+ * @buffer: a #PangoOTBuffer
+ *
+ * Empties a #PangoOTBuffer, make it ready to add glyphs to.
+ *
+ * Since: 1.4
+ **/ 
 void
 pango_ot_buffer_clear (PangoOTBuffer *buffer)
 {
@@ -64,17 +91,39 @@ pango_ot_buffer_clear (PangoOTBuffer *buffer)
   buffer->applied_gpos = FALSE;
 }
 
+/**
+ * pango_ot_buffer_add_glyph
+ * @buffer: a #PangoOTBuffer
+ * @glyph: the glyph index to add, like a #PangoGlyph
+ * @properties: the glyph properties
+ * @cluster: the cluster that this glyph belongs to
+ *
+ * Appends a glyph to a #PangoOTBuffer, with @properties identifying which
+ * features should be applied on this glyph.  See pango_ruleset_add_feature().
+ *
+ * Since: 1.4
+ **/ 
 void
 pango_ot_buffer_add_glyph (PangoOTBuffer *buffer,
-			   guint          glyph_index,
+			   guint          glyph,
 			   guint          properties,
 			   guint          cluster)
 {
   otl_buffer_add_glyph (buffer->buffer,
-			glyph_index, properties, cluster);
+			glyph, properties, cluster);
 
 }
 
+/**
+ * pango_ot_buffer_set_rtl
+ * @buffer: a #PangoOTBuffer
+ * @rtl: %TRUE for right-to-left text
+ *
+ * Sets whether glyphs will be rendered right-to-left.  This setting
+ * is needed for proper horizontal positioning of right-to-left scripts.
+ *
+ * Since: 1.4
+ **/ 
 void
 pango_ot_buffer_set_rtl (PangoOTBuffer *buffer,
 			 gboolean       rtl)
@@ -84,7 +133,7 @@ pango_ot_buffer_set_rtl (PangoOTBuffer *buffer,
 }
 
 /**
- * pango_ot_buffer_set_zero_width_marks:
+ * pango_ot_buffer_set_zero_width_marks
  * @buffer: a #PangoOTBuffer
  * @zero_width_marks: %TRUE if characters with a mark class should
  *  be forced to zero width.
@@ -93,6 +142,8 @@ pango_ot_buffer_set_rtl (PangoOTBuffer *buffer,
  * This setting is needed for proper positioning of Arabic accents,
  * but will produce incorrect results with standard OpenType indic
  * fonts.
+ *
+ * Since: 1.6
  **/
 void
 pango_ot_buffer_set_zero_width_marks (PangoOTBuffer     *buffer,
@@ -101,6 +152,18 @@ pango_ot_buffer_set_zero_width_marks (PangoOTBuffer     *buffer,
   buffer->zero_width_marks = zero_width_marks != FALSE;
 }
 
+/**
+ * pango_ot_buffer_get_glyphs
+ * @buffer: a #PangoOTBuffer
+ * @glyphs: location to store the array of glyphs, or %NULL
+ * @n_glyphs: location to store the number of glyphs, or %NULL
+ *
+ * Gets the glyph array contained in a #PangoOTBuffer.  The glyphs are
+ * owned by the buffer and should not be freed, and are only valid as long
+ * as buffer is not modified.
+ *
+ * Since: 1.4
+ **/ 
 void
 pango_ot_buffer_get_glyphs (PangoOTBuffer  *buffer,
 			    PangoOTGlyph  **glyphs,
@@ -203,6 +266,17 @@ apply_gpos_rtl (PangoGlyphString *glyphs,
     }
 }
 
+/**
+ * pango_ot_buffer_output
+ * @buffer: a #PangoOTBuffer
+ * @glyphs: a #PangoGlyphString
+ *
+ * Exports the glyphs in a #PangoOTBuffer into a #PangoGlyphString.  This is
+ * typically used after the OpenType layout processing is over, to convert the
+ * resulting glyphs into a generic Pango glyph string.
+ *
+ * Since: 1.4
+ **/ 
 void
 pango_ot_buffer_output (PangoOTBuffer    *buffer,
 			PangoGlyphString *glyphs)
