@@ -56,7 +56,7 @@ static PangoAttribute *pango_attr_size_new_internal (int                   size,
 
 /**
  * pango_attr_type_register:
- * @name: an identifier for the type. (Currently unused.)
+ * @name: an identifier for the type (currently unused.)
  * 
  * Allocate a new attribute type ID.
  * 
@@ -72,11 +72,12 @@ pango_attr_type_register (const gchar *name)
 
 /**
  * pango_attribute_copy:
- * @attr: a #PangoAttribute.
+ * @attr: a #PangoAttribute
  * 
  * Make a copy of an attribute.
  * 
- * Return value: a newly allocated #PangoAttribute.
+ * Return value: the newly allocated #PangoAttribute, which should be
+ *               freed with pango_attribute_destroy().
  **/
 PangoAttribute *
 pango_attribute_copy (const PangoAttribute *attr)
@@ -168,7 +169,8 @@ pango_attr_string_new (const PangoAttrClass *klass,
  * 
  * Create a new font family attribute.
  * 
- * Return value: the new #PangoAttribute.
+ * Return value: the newly allocated #PangoAttribute, which should be
+ *               freed with pango_attribute_destroy().
  **/
 PangoAttribute *
 pango_attr_family_new (const char *family)
@@ -210,7 +212,8 @@ pango_attr_language_equal (const PangoAttribute *attr1,
  * 
  * Create a new language tag attribute. 
  * 
- * Return value: the new #PangoAttribute.
+ * Return value: the newly allocated #PangoAttribute, which should be
+ *               freed with pango_attribute_destroy().
  **/
 PangoAttribute *
 pango_attr_language_new (PangoLanguage *language)
@@ -286,7 +289,8 @@ pango_attr_color_new (const PangoAttrClass *klass,
  * 
  * Create a new foreground color attribute.
  * 
- * Return value: the new #PangoAttribute.
+ * Return value: the newly allocated #PangoAttribute, which should be
+ *               freed with pango_attribute_destroy().
  **/
 PangoAttribute *
 pango_attr_foreground_new (guint16 red,
@@ -311,7 +315,8 @@ pango_attr_foreground_new (guint16 red,
  * 
  * Create a new background color attribute.
  * 
- * Return value: the new #PangoAttribute.
+ * Return value: the newly allocated #PangoAttribute, which should be
+ *               freed with pango_attribute_destroy().
  **/
 PangoAttribute *
 pango_attr_background_new (guint16 red,
@@ -458,7 +463,8 @@ pango_attr_size_new_internal (int size,
  * 
  * Create a new font-size attribute in fractional points.
  * 
- * Return value: the new #PangoAttribute.
+ * Return value: the newly allocated #PangoAttribute, which should be
+ *               freed with pango_attribute_destroy().
  **/
 PangoAttribute *
 pango_attr_size_new (int size)
@@ -472,7 +478,8 @@ pango_attr_size_new (int size)
  * 
  * Create a new font-size attribute in device units.
  * 
- * Return value: the new #PangoAttribute.
+ * Return value: the newly allocated #PangoAttribute, which should be
+ *               freed with pango_attribute_destroy().
  *
  * Since: 1.8
  **/
@@ -488,7 +495,8 @@ pango_attr_size_new_absolute (int size)
  * 
  * Create a new font slant style attribute.
  * 
- * Return value: the new #PangoAttribute.
+ * Return value: the newly allocated #PangoAttribute, which should be
+ *               freed with pango_attribute_destroy().
  **/
 PangoAttribute *
 pango_attr_style_new (PangoStyle style)
@@ -509,7 +517,8 @@ pango_attr_style_new (PangoStyle style)
  * 
  * Create a new font weight attribute.
  * 
- * Return value: the new #PangoAttribute.
+ * Return value: the newly allocated #PangoAttribute, which should be
+ *               freed with pango_attribute_destroy().
  **/
 PangoAttribute *
 pango_attr_weight_new (PangoWeight weight)
@@ -530,7 +539,8 @@ pango_attr_weight_new (PangoWeight weight)
  *
  * Create a new font variant attribute (normal or small caps)
  * 
- * Return value: the new #PangoAttribute.
+ * Return value: the newly allocated #PangoAttribute, which should be
+ *               freed with pango_attribute_destroy().
  **/
 PangoAttribute *
 pango_attr_variant_new (PangoVariant variant)
@@ -551,7 +561,8 @@ pango_attr_variant_new (PangoVariant variant)
  * 
  * Create a new font stretch attribute
  * 
- * Return value: the new #PangoAttribute.
+ * Return value: the newly allocated #PangoAttribute, which should be
+ *               freed with pango_attribute_destroy().
  **/
 PangoAttribute *
 pango_attr_stretch_new (PangoStretch  stretch)
@@ -595,13 +606,14 @@ pango_attr_font_desc_equal (const PangoAttribute *attr1,
 
 /**
  * pango_attr_font_desc_new:
- * @desc: 
+ * @desc: the font description
  * 
- * Create a new font description attribute. (This attribute
+ * Create a new font description attribute. This attribute
  * allows setting family, style, weight, variant, stretch,
- * and size simultaneously.)
+ * and size simultaneously.
  * 
- * Return value: 
+ * Return value:  the newly allocated #PangoAttribute, which should be
+ *               freed with pango_attribute_destroy().
  **/
 PangoAttribute *
 pango_attr_font_desc_new (const PangoFontDescription *desc)
@@ -625,9 +637,10 @@ pango_attr_font_desc_new (const PangoFontDescription *desc)
  * pango_attr_underline_new:
  * @underline: the underline style.
  * 
- * Create a new underline-style object.
+ * Create a new underline-style attribute.
  * 
- * Return value: the new #PangoAttribute.
+ * Return value: the newly allocated #PangoAttribute, which should be
+ *               freed with pango_attribute_destroy().
  **/
 PangoAttribute *
 pango_attr_underline_new (PangoUnderline underline)
@@ -652,7 +665,10 @@ pango_attr_underline_new (PangoUnderline underline)
  * modifies the color of underlines. If not set, underlines
  * will use the foreground color.
  * 
- * Return value: the new #PangoAttribute.
+ * Return value: the newly allocated #PangoAttribute, which should be
+ *               freed with pango_attribute_destroy().
+ *
+ * Since: 1.8
  **/
 PangoAttribute *
 pango_attr_underline_color_new (guint16 red,
@@ -673,9 +689,10 @@ pango_attr_underline_color_new (guint16 red,
  * pango_attr_strikethrough_new:
  * @strikethrough: %TRUE if the text should be struck-through.
  * 
- * Create a new font strike-through attribute.
+ * Create a new strike-through attribute.
  * 
- * Return value: the new #PangoAttribute.
+ * Return value: the newly allocated #PangoAttribute, which should be
+ *               freed with pango_attribute_destroy().
  **/
 PangoAttribute *
 pango_attr_strikethrough_new (gboolean strikethrough)
@@ -700,7 +717,10 @@ pango_attr_strikethrough_new (gboolean strikethrough)
  * modifies the color of strikethrough lines. If not set, strikethrough
  * lines will use the foreground color.
  * 
- * Return value: the new #PangoAttribute.
+ * Return value: the newly allocated #PangoAttribute, which should be
+ *               freed with pango_attribute_destroy().
+ *
+ * Since: 1.8
  **/
 PangoAttribute *
 pango_attr_strikethrough_color_new (guint16 red,
@@ -724,7 +744,8 @@ pango_attr_strikethrough_color_new (guint16 red,
  * 
  * Create a new baseline displacement attribute.
  * 
- * Return value: the new #PangoAttribute.
+ * Return value: the newly allocated #PangoAttribute, which should be
+ *               freed with pango_attribute_destroy().
  **/
 PangoAttribute *
 pango_attr_rise_new (int rise)
@@ -746,7 +767,8 @@ pango_attr_rise_new (int rise)
  * Create a new font size scale attribute. The base font for the
  * affected text will have its size multiplied by @scale_factor.
  * 
- * Return value: the new #PangoAttribute.
+ * Return value: the newly allocated #PangoAttribute, which should be
+ *               freed with pango_attribute_destroy().
  **/
 PangoAttribute*
 pango_attr_scale_new (double scale_factor)
@@ -773,7 +795,10 @@ pango_attr_scale_new (double scale_factor)
  * other fonts on the system that might contain the characters in the
  * text.
  * 
- * Return value: the new #PangoAttribute.
+ * Return value: the newly allocated #PangoAttribute, which should be
+ *               freed with pango_attribute_destroy().
+ *
+ * Since: 1.4
  **/
 PangoAttribute *
 pango_attr_fallback_new (gboolean enable_fallback)
@@ -795,7 +820,8 @@ pango_attr_fallback_new (gboolean enable_fallback)
  * 
  * Create a new letter-spacing attribute.
  * 
- * Return value: the new #PangoAttribute.
+ * Return value: the newly allocated #PangoAttribute, which should be
+ *               freed with pango_attribute_destroy().
  *
  * Since: 1.6
  **/
@@ -859,19 +885,20 @@ pango_attr_shape_equal (const PangoAttribute *attr1,
 /**
  * pango_attr_shape_new_with_data:
  * @ink_rect:     ink rectangle to assign to each character
- * @logical_rect: logical rectangle assign to each character
+ * @logical_rect: logical rectangle to assign to each character
  * @data:         user data pointer
  * @copy_func:    function to copy @data when the attribute
  *                is copied. If %NULL, @data is simply copied
  *                as a pointer.
  * @destroy_func: function to free @data when the attribute
- *                is freed, or %NULL.
+ *                is freed, or %NULL
  * 
  * Like pango_attr_shape_new(), but a user data pointer is also
- * provided; this pointer can be accessed when rendering later
+ * provided; this pointer can be accessed when later
  * rendering the glyph.
  * 
- * Return value: the newly created attribute
+ * Return value: the newly allocated #PangoAttribute, which should be
+ *               freed with pango_attribute_destroy().
  *
  * Since: 1.8
  **/
@@ -908,14 +935,15 @@ pango_attr_shape_new_with_data (const PangoRectangle  *ink_rect,
 /**
  * pango_attr_shape_new:
  * @ink_rect:     ink rectangle to assign to each character
- * @logical_rect: logical rectangle assign to each character
+ * @logical_rect: logical rectangle to assign to each character
  * 
  * Create a new shape attribute. A shape is used to impose a
  * particular ink and logical rect on the result of shaping a
  * particular glyph. This might be used, for instance, for
  * embedding a picture or a widget inside a PangoLayout.
  * 
- * Return value: the newly created attribute
+ * Return value: the newly allocated #PangoAttribute, which should be
+ *               freed with pango_attribute_destroy().
  **/
 PangoAttribute *
 pango_attr_shape_new (const PangoRectangle *ink_rect,
@@ -944,9 +972,10 @@ pango_attr_list_get_type (void)
 /**
  * pango_attr_list_new:
  * 
- * Create a new empty attribute list with a reference count of 1.
+ * Create a new empty attribute list with a reference count of one.
  * 
- * Return value: the newly allocated #PangoAttrList.
+ * Return value: the newly allocated #PangoAttrList, which should
+ *               be freed with pango_attr_list_unref().
  **/
 PangoAttrList *
 pango_attr_list_new (void)
@@ -965,13 +994,19 @@ pango_attr_list_new (void)
  * @list: a #PangoAttrList
  * 
  * Increase the reference count of the given attribute list by one.
+ *
+ * Return value: The attribute list passed in
+ *
+ * Since: 1.10
  **/
-void
+PangoAttrList *
 pango_attr_list_ref (PangoAttrList *list)
 {
-  g_return_if_fail (list != NULL);
+  g_return_val_if_fail (list != NULL, NULL);
 
   list->ref_count++;
+
+  return list;
 }
 
 /**
@@ -1012,9 +1047,11 @@ pango_attr_list_unref (PangoAttrList *list)
  * pango_attr_list_copy:
  * @list: a #PangoAttrList
  * 
- * Copy @list and return an identical, new list.
+ * Copy @list and return an identical new list.
  *
- * Return value: new attribute list
+ * Return value: the newly allocated #PangoAttrList, with a
+ *               reference count of one, which should
+ *               be freed with pango_attr_list_unref().
  **/
 PangoAttrList *
 pango_attr_list_copy (PangoAttrList *list)
@@ -1399,7 +1436,7 @@ pango_attr_list_splice (PangoAttrList *list,
 	   * an element could (in theory) suprise the caller, so
 	   * we don't delete it.
 	   */
-	  attr->start_index = CLAMP_ADD (attr->end_index, ulen);
+	  attr->start_index = CLAMP_ADD (attr->start_index, ulen);
 	  attr->end_index = CLAMP_ADD (attr->end_index, ulen);
 	}
 
@@ -1420,6 +1457,7 @@ pango_attr_list_splice (PangoAttrList *list,
       
       tmp_list = tmp_list->next;
     }
+#undef CLAMP_ADD
 }
 
 /**
@@ -1427,9 +1465,10 @@ pango_attr_list_splice (PangoAttrList *list,
  * @list: a #PangoAttrList
  * 
  * Create a iterator initialized to the beginning of the list.
+ * @list must not be modified until this iterator is freed.
  * 
- * Return value: a new #PangoIterator. @list must not be modified
- *               until this iterator is freed with pango_attr_iterator_destroy().
+ * Return value: the newly allocated #PangoAttrIterator, which should
+ *               be freed with pango_attr_iterator_destroy().
  **/
 PangoAttrIterator *
 pango_attr_list_get_iterator (PangoAttrList  *list)
@@ -1539,7 +1578,8 @@ pango_attr_iterator_next (PangoAttrIterator *iterator)
  * 
  * Copy a #PangoAttrIterator
  *
- * Return value: Copy of @iterator
+ * Return value: the newly allocated #PangoAttrIterator, which should
+ *               be freed with pango_attr_iterator_destroy().
  **/
 PangoAttrIterator *
 pango_attr_iterator_copy (PangoAttrIterator *iterator)
@@ -1613,7 +1653,7 @@ pango_attr_iterator_get (PangoAttrIterator *iterator,
  * @iterator: a #PangoAttrIterator
  * @desc: a #PangoFontDescription to fill in with the current values.
  *        The family name in this structure will be set using
- *        pango_font_description_set_family_static using values from
+ *        pango_font_description_set_family_static() using values from
  *        an attribute in the #PangoAttrList associated with the iterator,
  *        so if you plan to keep it around, you must call:
  *        <literal>pango_font_description_set_family (desc, pango_font_description_get_family (desc))</literal>.
@@ -1773,8 +1813,10 @@ pango_attr_iterator_get_font (PangoAttrIterator     *iterator,
  * of @list for which @func returns %TRUE and inserts them into
  * a new list.
  * 
- * Return value: a newly allocated #PangoAttrList or %NULL if
+ * Return value: the new #PangoAttrList or %NULL if
  *  no attributes of the given types were found.
+ *
+ * Since: 1.2
  **/
 PangoAttrList *
 pango_attr_list_filter (PangoAttrList       *list,
@@ -1842,6 +1884,8 @@ pango_attr_list_filter (PangoAttrList       *list,
  * Return value: a list of all attributes for the current range.
  *   To free this value, call pango_attribute_destroy() on
  *   each value and g_slist_free() on the list.
+ *
+ * Since: 1.2
  **/
 GSList *
 pango_attr_iterator_get_attrs (PangoAttrIterator *iterator)
