@@ -74,8 +74,8 @@ pango_cairo_font_map_get_type (void)
  * You generally should only use the #PangoFontMap and
  * #PangoCairoFontMap interfaces on the returned object.
  * 
- * Return value: the newly created fontmap object. Free
- * with g_object_unref().
+ * Return value: the newly allocated #PangoFontMap, which should
+ *               be freed with g_object_unref().
  *
  * Since: 1.10
  **/
@@ -213,7 +213,7 @@ free_context_info (PangoCairoContextInfo *info)
   if (info->merged_options)
     cairo_font_options_destroy (info->merged_options);
   
-  free (info);
+  g_free (info);
 }
 
 static PangoCairoContextInfo *
@@ -372,7 +372,7 @@ pango_cairo_context_set_font_options (PangoContext               *context,
 }
   
 /**
- * pango_cairo_get_font_options:
+ * pango_cairo_context_get_font_options:
  * @context: a #PangoContext, from pango_cairo_font_map_create_context()
  * 
  * Retrieves any font rendering options previously set with
