@@ -58,7 +58,7 @@ typedef struct _PangoFcFontClass PangoFcFontClass;
 /**
  * PangoFcFont:
  * 
- * #PangoFcFontMap is a base class for font implementations
+ * #PangoFcFont is a base class for font implementations
  * using the FontConfig and FreeType libraries and is used in
  * conjunction with #PangoFcFontMap. When deriving from this
  * class, you need to implement all of its virtual functions
@@ -71,6 +71,8 @@ struct _PangoFcFont
 
   FcPattern *font_pattern;	    /* fully resolved pattern */
   PangoFontMap *fontmap;	    /* associated map */
+  gpointer context_key;		    /* used internally */
+  PangoMatrix matrix;		    /* used internally */
   PangoFontDescription *description;
   
   GSList *metrics_by_lang;
@@ -93,7 +95,7 @@ struct _PangoFcFont
  *   Unicode character.
  * @get_unknown_glyph: Gets the glyph that should be used to
  *   display an unknown-glyph indication for the specified
- *   unicode character.
+ *   Unicode character.
  * @shutdown: Performs any font-specific shutdown code that
  *   needs to be done when pango_fc_font_map_shutdown is called.
  *   May be %NULL.
